@@ -10,6 +10,7 @@ DROP TABLE orders;
 DROP TABLE movies;
 DROP TABLE subscribers;
 DROP TABLE booleantests;
+DROP TABLE defaults;
 DROP TABLE auto_id_tests;
 DROP TABLE entrants;
 DROP TABLE colnametests;
@@ -28,3 +29,6 @@ DROP TABLE fk_test_has_fk;
 DROP TABLE fk_test_has_pk;
 DROP TABLE keyboards;
 DROP TABLE legacy_things;
+DROP TABLE numeric_data;
+DROP TABLE [order];
+DROP TABLE mixed_case_monkeys;
